@@ -153,8 +153,11 @@ class EfiVar:
                  count = 0,
                  pkidx = 0,
                  authdata = None):
-        self.name = name
-        self.guid = guid
+        if isinstance(name, str):
+            self.name = ucs16.from_string(name)
+        else:
+            self.name = name
+        self.guid = guids.parse_auto(guid)
         self.attr = attr
         self.data = data
         self.count = count
@@ -248,6 +251,13 @@ class EfiVar:
         self.data = bytes(self.sigdb)
         self.update_time()
 
+    def sigdb_add_external_mgmt(self, guid):
+        if self.sigdb is None:
+            raise RuntimeError
+        self.sigdb.add_external_mgmt(guid)
+        self.data = bytes(self.sigdb)
+        self.update_time(initial_ts)
+
     def sigdb_add_dummy(self, guid):
         if self.sigdb is None:
             raise RuntimeError
@@ -255,6 +265,15 @@ class EfiVar:
         self.data = bytes(self.sigdb)
         self.update_time(initial_ts)
 
+    def sigdb_merge(self, mdb):
+        if self.sigdb is None:
+            raise RuntimeError
+        if mdb.sigdb is None:
+            raise RuntimeError
+        self.sigdb.merge(mdb.sigdb)
+        self.data = bytes(self.sigdb)
+        self.update_time(initial_ts)
+
     def set_bool(self, value):
         if value:
             self.data = b'\x01'
@@ -438,6 +457,13 @@ class EfiVarList(collections.UserDict):
         var.sigdb_add_hash(guids.parse_str(owner),
                            bytes.fromhex(hashdata))
 
+    def add_external_mgmt(self, owner):
+        var = self.get('PK')
+        if not var:
+            var = self.create('PK')
+        logging.info('add external management guid to PK')
+        var.sigdb_add_external_mgmt(owner)
+
     def add_dummy_dbx(self, owner):
         var = self.get('dbx')
         if var:
@@ -457,10 +483,7 @@ class EfiVarList(collections.UserDict):
                                   ms_kek = 'all'):
         self.add_cert('PK', guid, cert, True)
         self.add_cert('KEK', guid, cert, True)
-        if ms_kek in ('2011', 'all'):
-            self.add_cert('KEK', guids.MicrosoftVendor, certs.MS_KEK_2011, False)
-        if ms_kek in ('2023', 'all'):
-            self.add_cert('KEK', guids.MicrosoftVendor, certs.MS_KEK_2023, False)
+        self.add_microsoft_kek_keys(ms_kek)
         self.add_dummy_dbx(guids.OvmfEnrollDefaultKeys)
 
     def enroll_platform_redhat(self, ms_kek = 'all'):
@@ -470,11 +493,28 @@ class EfiVarList(collections.UserDict):
         pk = certs.pk_generate(cn = name)
         self.enroll_platform_with_cert(pk.name, ms_kek = ms_kek)
 
-    def add_microsoft_keys(self):
-        self.add_cert('db', guids.MicrosoftVendor, certs.MS_WIN_2011, False)
-        self.add_cert('db', guids.MicrosoftVendor, certs.MS_WIN_2023, False)
-        self.add_cert('db', guids.MicrosoftVendor, certs.MS_3RD_2011, False)
-        self.add_cert('db', guids.MicrosoftVendor, certs.MS_3RD_2023, False)
+    def enroll_managed_platform(self, ms_kek = 'all'):
+        self.add_external_mgmt(guids.OvmfEnrollDefaultKeys)
+        self.add_microsoft_kek_keys(ms_kek)
+        self.add_dummy_dbx(guids.OvmfEnrollDefaultKeys)
+
+    def add_microsoft_keys(self, ms_db = 'all'):
+        if ms_db in ('win11', 'all'):
+            self.add_cert('db', guids.MicrosoftVendor, certs.MS_WIN_2011, False)
+        if ms_db in ('win11', 'win23', 'win+rom23', 'all'):
+            self.add_cert('db', guids.MicrosoftVendor, certs.MS_WIN_2023, False)
+        if ms_db in ('uefi11', 'all'):
+            self.add_cert('db', guids.MicrosoftVendor, certs.MS_3RD_2011, False)
+        if ms_db in ('uefi11', 'uefi23', 'uefi+rom23', 'all'):
+            self.add_cert('db', guids.MicrosoftVendor, certs.MS_3RD_2023, False)
+        if ms_db in ('win+rom23', 'uefi+rom23'):
+            self.add_cert('db', guids.MicrosoftVendor, certs.MS_ROM_2023, False)
+
+    def add_microsoft_kek_keys(self, ms_kek = 'all'):
+        if ms_kek in ('2011', 'all'):
+            self.add_cert('KEK', guids.MicrosoftVendor, certs.MS_KEK_2011, False)
+        if ms_kek in ('2023', 'all'):
+            self.add_cert('KEK', guids.MicrosoftVendor, certs.MS_KEK_2023, False)
 
     def add_distro_keys(self, distro):
         certlist = certs.DISTRO_CA.get(distro).get('certs')
@@ -486,7 +526,7 @@ class EfiVarList(collections.UserDict):
 
 
     @staticmethod
-    def print_siglist(slist):
+    def print_siglist(slist, print_hashes = True):
         name = guids.name(slist.guid)
         count = len(slist)
         print(f'  siglist type={name} count={count}')
@@ -501,7 +541,7 @@ class EfiVarList(collections.UserDict):
                 print(f'    issuer CN={icn.value}')
             except IndexError:
                 print('    no issuer CN')
-        elif str(slist.guid) == guids.EfiCertSha256:
+        elif str(slist.guid) == guids.EfiCertSha256 and print_hashes:
             for sig in list(slist):
                 print(f"    {sig['data'].hex()}")
 
@@ -532,7 +572,7 @@ class EfiVarList(collections.UserDict):
         if count < len(data):
             print(f'  {pos:06x}:  [ ... ]')
 
-    def print_normal(self, hexdump = False):
+    def print_normal(self, hexdump = False, print_hashes = True):
         for (key, var) in sorted(self.items()):
             name = str(var.name)
             gname = guids.name(var.guid)
@@ -550,7 +590,7 @@ class EfiVarList(collections.UserDict):
                 print('  ' + line)
             if var.sigdb:
                 for slist in var.sigdb:
-                    self.print_siglist(slist)
+                    self.print_siglist(slist, print_hashes)
             if hexdump:
                 self.print_hexdump(var.data)
             print('')
