@@ -30,7 +30,7 @@ class EfiSigList(collections.UserList):
         if data:
             self.parse_bin(data)
         elif guid:
-            self.guid = guid
+            self.guid = guids.parse_auto(guid)
             if header:
                 self.header = header
 
@@ -59,7 +59,7 @@ class EfiSigList(collections.UserList):
                 except ValueError:
                     logging.error("x509: failed to load certificate")
                     self.x509 = None
-        sig = { 'guid' : guid,
+        sig = { 'guid' : guids.parse_auto(guid),
                 'data' : data }
         self.append(sig)
 
@@ -132,14 +132,18 @@ class EfiSigDB(collections.UserList):
             self.append(siglist)
             pos += siglist.size()
 
-    def add_cert(self, guid, filename):
-        siglist = EfiSigList(guid = guids.parse_str(guids.EfiCertX509))
-        siglist.add_cert(guid, filename)
+    def has_cert(self, siglist):
         for item in list(self):
             if item.x509 == siglist.x509:
                 logging.info('certificate already present, skipping')
-                return
-        self.append(siglist)
+                return True
+        return False
+
+    def add_cert(self, guid, filename):
+        siglist = EfiSigList(guid = guids.parse_str(guids.EfiCertX509))
+        siglist.add_cert(guid, filename)
+        if not self.has_cert(siglist):
+            self.append(siglist)
 
     def has_sig(self, sigtype, sigdata):
         for siglist in list(self):
@@ -171,6 +175,21 @@ class EfiSigDB(collections.UserList):
         siglist.add_sig(guid, hashlib.sha256(b'').digest())
         self.append(siglist)
 
+    def add_external_mgmt(self, guid):
+        data = b'\0'
+        if not self.has_sig(guids.EfiCertExternalMgmt, data):
+            siglist = self.get_siglist(guids.EfiCertExternalMgmt)
+            siglist.add_sig(guid, data)
+
+    def merge(self, mdb):
+        for siglist in list(mdb):
+            if str(siglist.guid) == guids.EfiCertSha256:
+                for item in list(siglist):
+                    self.add_hash(item['guid'], item['data'])
+            if str(siglist.guid) == guids.EfiCertX509:
+                if not self.has_cert(siglist):
+                    self.append(siglist)
+
     def extract_certs(self, prefix = None):
         for siglist in list(self):
             siglist.extract_cert(prefix)
